From b03488c0fb28b74a490f164b04705a24f759e779 Mon Sep 17 00:00:00 2001
From: Heikki Linnakangas <heikki.linnakangas@iki.fi>
Date: Thu, 12 Feb 2015 15:37:19 +0200
Subject: [PATCH 2/3] Also drain input buffer in non-blocking mode if send
 fails with EAGAIN.

In blocking mode, pqSendSome() calls pqReadData() to read any NOTICEs and
such that the server may have sent, before sleeping. That avoids a deadlock
e.g. in COPY FROM STDIN, because the server might block on sending the
NOTICEs and not receive our data until we read the NOTICEs. But
non-blocking mode has exactly the same issue, so let's call pqReadData() in
non-blocking mode too.

Reviewed by Andres Freund

diff --git a/src/interfaces/libpq/fe-misc.c b/src/interfaces/libpq/fe-misc.c
index 9dd02d5..f0ab6cd 100644
--- a/src/interfaces/libpq/fe-misc.c
+++ b/src/interfaces/libpq/fe-misc.c
@@ -905,16 +905,6 @@ pqSendSome(PGconn *conn, int len)
 			/*
 			 * We didn't send it all, wait till we can send more.
 			 *
-			 * If the connection is in non-blocking mode we don't wait, but
-			 * return 1 to indicate that data is still pending.
-			 */
-			if (pqIsnonblocking(conn))
-			{
-				result = 1;
-				break;
-			}
-
-			/*
 			 * There are scenarios in which we can't send data because the
 			 * communications channel is full, but we cannot expect the server
 			 * to clear the channel eventually because it's blocked trying to
@@ -935,6 +925,17 @@ pqSendSome(PGconn *conn, int len)
 				result = -1;	/* error message already set up */
 				break;
 			}
+
+			/*
+			 * If the connection is in non-blocking mode we don't wait, but
+			 * return 1 to indicate that data is still pending.
+			 */
+			if (pqIsnonblocking(conn))
+			{
+				result = 1;
+				break;
+			}
+
 			if (pqWait(TRUE, TRUE, conn))
 			{
 				result = -1;
-- 
2.1.4

